/*
 * ClientDBAdapter.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.client.db;

import java.net.MalformedURLException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import ro.raducora.connection.client.AdapterBase;
import ro.raducora.utils.protocol.ConnectInfo;
import ro.raducora.utils.protocol.RemoteException;
import ro.raducora.utils.protocol.SerializedResultSet;
import ro.raducora.utils.props.ClientDBProps;
import ro.raducora.utils.io.Write;

/**
 * @author Radu Coravu
 * 
 * <br>
 * An adapter which connects to a database using JDBC
 */
public class ClientDBAdapter implements AdapterBase {

  //the used connection
  private Connection connection = null;

  //the used statement
  private Statement statement = null;

  private boolean isConnected = false;

  private static ClientDBProps props = null;

  /**
   * Load Properties
   * 
   * @param props
   *          Client DB properties
   */
  public ClientDBAdapter(ClientDBProps props) {
    ClientDBAdapter.props = props;
  }

  private ResultSet selectQuery(String sqlQuery) throws SQLException {
    ResultSet ret = null;
    statement = connection.createStatement();
    ret = statement.executeQuery(sqlQuery);
    return ret;
  }

  private int updateQuery(String sqlQuery) throws SQLException {
    int ret = -1;
    statement = connection.createStatement();
    ret = statement.executeUpdate(sqlQuery);
    return ret;
  }

  /**
   * Sends a query and returns the resulting Object(ResultSet or Integer)
   * 
   * @see ro.raducora.connection.client.AdapterBase#queryObject(java.lang.Object)
   */
  public synchronized Object queryObject(Object sent) throws SQLException {
    if (sent instanceof String) {
      String query = ((String) sent).trim();
      SerializedResultSet rs = null;
      if (!query.toUpperCase().startsWith("SELECT") && !query.toUpperCase().startsWith("SHOW")) {
        int i = updateQuery(query);
        statement.close();
        statement = null;
        return new Integer(i);
      } else {
        ResultSet rSet = selectQuery(query);
        rs = new SerializedResultSet(rSet);
        rSet.close();
        statement.close();
        statement = null;
        return rs;
      }
    }
    return null;
  }

  /**
   * Returns true if connected
   * 
   * @return true if connected
   */
  public boolean isConnected() {
    return isConnected;
  }

  /**
   * Connect to database
   * 
   * @param info
   *          the connection info
   */
  public void connect(ConnectInfo info) throws Exception {
    String connectionString = complete(props.getServerSubprotocol(info.getDBName()), props
        .getServerHostname(info.getDBName()), props.getRealDBName(info.getDBName()), props
        .getDriverClassName(info.getDBName()), props.getDriverJarPath(info.getDBName()), props
        .getDriverJarFilename(info.getDBName()));
    connection = BrokerHandle.getConnectionManagerBroker().getPoolsManager().getConnectionPool(
        info, props, connectionString).getConnection();
    isConnected = true;
  }

  private String complete(String subprotocol, String hostname, String dbname,
      String driverClassName, String pathToJar, String jarFilename) throws RemoteException,
      SQLException, MalformedURLException, ClassNotFoundException, InstantiationException,
      IllegalAccessException {
    hostname = hostname.trim();
    if (!hostname.equalsIgnoreCase("")) {
      hostname = "//" + hostname + "/";
    }
    String connectionString = "jdbc:" + subprotocol + ":" + hostname + dbname;

    //maybe driver already loaded
    try {
      DriverManager.getDriver("jdbc:" + subprotocol + ":" + hostname);
    } catch (SQLException e) {
      //driver not loaded yet
      Write.Line("Loading new Driver---");
      //a jar filename is present
      //load from jar
      if ((!jarFilename.trim().equals(""))) {
        JarDriverLoader.loadDriver(pathToJar, jarFilename, driverClassName);
      } else if (!driverClassName.trim().equals("")) {
        //load from classpath
        Class.forName(driverClassName);
      }
      Write.Line("Connection URL:" + connectionString);
    }
    return connectionString;
  }

  /**
   * Disconnect
   */
  public void disconnect() throws SQLException {
    isConnected = false;
    if (connection != null && (!connection.isClosed())) {
      try {
        connection.close();
        connection = null;
      } catch (SQLException exception) {
      }
    }
  }

  /**
   * Reload ClientDBProps properties
   */
  public void reloadProperties() throws Exception {
    props.reload();
  }

  /**
   * Is Cloneable
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone();
  }

  /**
   * Return false;
   */
  public boolean isAbstractConnection() {
    return false;
  }

  /**
   * Gets the connection status returned by the <code>connect</code> method
   * 
   * @return the connection status
   */
  public Object getConnectionStatus() {
    return isConnected ? "true" : "false";
  }
}
