/*
 * ConnectionPool.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.client.db;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Iterator;
import java.util.List;

import ro.raducora.utils.io.Write;

/**
 * @author Radu Coravu
 * 
 * <br>
 * Implementation of a Connection Pool for database connections.
 */
class ConnectionPool {

  private List availableConnections = Collections.synchronizedList(new ArrayList());

  private List usedConnections = Collections.synchronizedList(new ArrayList());

  private int maxNoConnections = 0;

  private int minConnections = 0;

  private String user = null;

  private String password = null;

  private String url = null;

  /**
   * Create new Connection Pool
   * 
   * @param dbUrl
   *          the database url
   * @param user
   *          the username
   * @param password
   *          the password
   * @param maxNoConnections
   *          the total no of connections
   * @param minConnections
   *          the min no of connections
   * 
   * @throws SQLException
   */
  public ConnectionPool(String dbUrl, String user, String password, int maxNoConnections,
      int minConnections) throws SQLException {
    this.url = dbUrl;
    this.user = user;
    this.password = password;
    this.maxNoConnections = maxNoConnections;
    this.minConnections = minConnections;
    this.init();
    Write.Line("New Connection Pool Created " + this);
  }

  /**
   * Creates the Min no of PooledConnections
   * 
   * @throws SQLException
   */
  protected void init() throws SQLException {
    for (int i = 0; i < minConnections; i++) {
      PooledConnection con = createConnection();
      availableConnections.add(con);
    }
  }

  /**
   * Gets a Connection from Pool or creates it if necessary
   * 
   * @return the ConnectionPool
   * 
   * @throws SQLException
   */
  public synchronized Connection getConnection() throws SQLException {
    PooledConnection con = null;
    if (availableConnections.size() > 0) {
      con = (PooledConnection) availableConnections.get(0);
      availableConnections.remove(0);
      usedConnections.add(con);
      try {
        if (con.isClosed()) {
          con = createConnection();
        }
      } catch (SQLException e) {
        this.freeConnection(con);
        throw new SQLException(
            "ConnectionPool:getConnection()::Connection Problem -- SQLException: " + e.toString());
      }
    } else {
      if ((availableConnections.size() + usedConnections.size()) < maxNoConnections) {
        try {
          con = createConnection();
          usedConnections.add(con);
        } catch (SQLException exception) {
          throw new SQLException(
              "ConnectionPool:getConnection()::Cannot create new connection -- Exception:"
                  + exception.toString());
        }
      } else {
        throw new SQLException("Cannot open new connection:");
      }
    }
    return con;
  }

  /**
   * Check available connections if alive If alive,keep them Else make new
   * connections ,add them to available
   * 
   * @param difference
   *          if (the oldest time stamp -current time stamp) > difference then
   *          refresh connection.
   */
  public synchronized void refreshConnections(long difference) throws SQLException {
    synchronized (availableConnections) {
      Iterator iter = availableConnections.iterator();
      long currentTime = System.currentTimeMillis();
      for (; iter.hasNext();) {
        PooledConnection connection = (PooledConnection) iter.next();
        //only if connection hasn't been used for 30 minutes
        if ((currentTime - connection.getLastCheckedStamp()) > difference) {
          try {
            if ((currentTime - connection.getLastCheckedStamp()) > 100 * difference) {
              //:) create a new connection once in a while
              throw new SQLException("Catch Me");
            } else {
              //test if connection is really up
              Statement statement = connection.createStatement();
              statement.executeQuery("select 1").close();
              statement.close();
              Write.Line("Pooled Connection Refreshed");
            }
          } catch (SQLException e) {
            connection = createConnection();
          } finally {
            connection.setLastCheckedStamp(System.currentTimeMillis());
          }
        }
      }
    }
  }

  /**
   * Free a Connection(Adds it to available,removes it from used)
   * 
   * @param con
   *          the connection to set as unused
   */
  public synchronized void freeConnection(Connection con) {
    int index = usedConnections.indexOf(con);
    Write.Line("Free Connection number " + index);
    if (index >= 0) {
      availableConnections.add(con);
      usedConnections.remove(index);
    }
  }

  /**
   * Creates new Connection
   * 
   * @return the new connection
   * @throws SQLException
   */
  protected PooledConnection createConnection() throws SQLException {
    PooledConnection con = null;
    try {
      con = new PooledConnection(DriverManager.getConnection(this.url, this.user, this.password),
          this);

    } catch (SQLException e) {
      throw new SQLException("ConnectionPool:createConnection::SQLException " + e.toString());
    }
    return con;
  }

  /**
   * Closes all Available Connections
   */
  public synchronized void closeAvailableConnections() {
    closeConnections(availableConnections);
  }

  /**
   * Closes all Used Connections
   */
  public synchronized void closeUsedConnections() {
    closeConnections(usedConnections);
  }

  /**
   * Close all Connections
   */
  public synchronized void closeAllConnections() {
    closeAvailableConnections();
    closeUsedConnections();
    availableConnections.clear();
    usedConnections.clear();
  }

  /**
   * Closes available connections to minimum number of connections for pool
   * 
   * @param difference
   *          if (the oldest time stamp -current time stamp) > difference then
   *          close connection.
   */
  public synchronized void closeAvailableConnectionsToMinimumNo(long difference) {
    synchronized (availableConnections) {
      int currentNo = availableConnections.size() + usedConnections.size();
      long currentTime = System.currentTimeMillis();
      while (currentNo > minConnections) {
        if (availableConnections.size() > 0) {
          PooledConnection conn = (PooledConnection) availableConnections.get(0);
          if ((currentTime - conn.getLastCheckedStamp()) > difference) {
            try {
              conn.cleanUpConnection();
              Write.Line("Closed Connection::" + currentNo);
            } catch (SQLException e) {
              Write.exception(e);
            }
          }
          availableConnections.remove(0);
          currentNo--;
        } else {
          return;
        }
      }
    }
  }

  /**
   * Closes all Connections
   * 
   * Clear all Connections Lists
   */
  public synchronized void destroy() {
    closeAllConnections();
    availableConnections.clear();
    usedConnections.clear();
  }

  /**
   * Close a List of Connections
   * 
   * @param aList
   *          the list
   */
  protected synchronized void closeConnections(List aList) {
    for (int i = 0; i < aList.size(); i++) {
      PooledConnection con = (PooledConnection) aList.get(i);
      try {
        con.cleanUpConnection();
      } catch (Exception e) {
        Write.exception(e);
      }
    }
  }

  //used for tests
  int getUsedConnNumber() {
    return usedConnections.size();
  }

  //used for tests
  int getAvailConnNumber() {
    return availableConnections.size();
  }

}
