/*
 * ConnectionPoolManager.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.client.db;

import java.sql.SQLException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import ro.raducora.utils.io.Write;
import ro.raducora.utils.props.ClientDBProps;
import ro.raducora.utils.protocol.ConnectInfo;

/**
 * Class for getting pooled connections from ConnectionPool.<br>
 * 
 * @author Radu Coravu
 *  
 */
class ConnectionPoolManager {

  //the map with all connection pools
  private Map connectionPools = new HashMap(10);

  /**
   * Gets the Connection Pool for specified parameters
   * 
   * @param info
   *          -ConnectInfo object containing db name,username,password
   * @param props
   *          db adapter properties
   * @param dbURL
   *          URL to connect to
   * @return a ConnectionPool
   * @throws SQLException
   */
  public synchronized ConnectionPool getConnectionPool(ConnectInfo info, ClientDBProps props,
      String dbURL) throws SQLException {

    //make a hashcode to use as a Pool ID to check if the Pool is already
    // created
    Integer poolHash = new Integer(
        ((props.getRealDBName(info.getDBName()) + info.getUsername() + info.getPassword()))
            .hashCode());
    ConnectionPool pool = (ConnectionPool) connectionPools.get(poolHash);

    if (pool != null) { return pool; }
    //makes a new ConnectionPool
    pool = new ConnectionPool(dbURL, info.getUsername(), info.getPassword(), props
        .getMaxNoConn(info.getDBName()), props.getMinNoConn(info.getDBName()));

    //Add digest to key map
    connectionPools.put(poolHash, pool);

    return pool;
  }

  /**
   * Closes available connections for all managed pools to minimum. It goes
   * through all pools ,and checks if creation stamp for a pooled connection is
   * older than the current time stamp with more than the given difference. If
   * it is,the pooled connection is closed ,if the available connections are
   * more than the min number of connections in the pool.
   * 
   * @param difference
   *          The time difference.
   */
  public synchronized void closeAvailableConnectionsToMinimumNo(long difference) {
    synchronized (connectionPools) {
      Iterator iterator = connectionPools.values().iterator();
      while (iterator.hasNext()) {
        ConnectionPool pool = ((ConnectionPool) iterator.next());
        try {
          pool.closeAvailableConnectionsToMinimumNo(4 * difference);
        } catch (Exception e) {
          Write.exception(e);
        }
        Write.Line("Pool:" + pool + " Cleaned");
      }
    }
  }

  /**
   * Checks connections for all managed pools if they are alive. It goes
   * through all pools ,and checks if creation stamp for a pooled connection is
   * older than the current time stamp with more than the given difference. If
   * it is,the pooled connection is checked (and thus refreshed) with an SQL
   * Query. If it has been closed,it is again created.
   * 
   * @param difference
   *          The time difference.
   */
  public synchronized void refreshConnections(long difference) throws SQLException {
    synchronized (connectionPools) {
      Iterator iterator = connectionPools.values().iterator();
      while (iterator.hasNext()) {
        ConnectionPool pool = ((ConnectionPool) iterator.next());
        try {
          pool.refreshConnections(difference);
        } catch (Exception e) {
          Write.exception(e);
        }
      }
      Write.Line("Connections Refreshed");
    }
  }
}