/*
 * PooledConnection.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.client.db;

import java.sql.Connection;
import java.sql.Statement;
import java.sql.PreparedStatement;
import java.sql.CallableStatement;
import java.sql.DatabaseMetaData;
import java.sql.Savepoint;
import java.sql.SQLWarning;
import java.sql.SQLException;

/**
 * @author Radu Coravu <br>
 *         Implementation of a database connection that cannot be physically
 *         closed. All calls to close() will return this connection to the pool
 *         registered with this connection.
 */

class PooledConnection implements Connection {

  private Connection con = null;

  private ConnectionPool pool = null;

  //last time stamp when conection checked
  private long checkedOn;

  /**
   * Pooled Connection Constructor
   * 
   * @param con
   *          the wrapped connection
   * @param pool
   *          the connection pool
   */
  public PooledConnection(Connection con, ConnectionPool pool) {
    this.con = con;
    this.pool = pool;
  }

  /**
   * @see java.sql.Connection#createStatement()
   */
  public Statement createStatement() throws SQLException {
    return con.createStatement();
  }

  /**
   * @see java.sql.Connection#prepareStatement(java.lang.String)
   */
  public PreparedStatement prepareStatement(String sql) throws SQLException {
    return con.prepareStatement(sql);
  }

  /**
   * @see java.sql.Connection#prepareCall(java.lang.String)
   */
  public CallableStatement prepareCall(String sql) throws SQLException {
    return con.prepareCall(sql);
  }

  /**
   * @see java.sql.Connection#nativeSQL(java.lang.String)
   */
  public String nativeSQL(String sql) throws SQLException {
    return con.nativeSQL(sql);
  }

  /**
   * @see java.sql.Connection#setAutoCommit(boolean)
   */
  public void setAutoCommit(boolean b) throws SQLException {
    con.setAutoCommit(b);
  }

  /**
   * @see java.sql.Connection#getAutoCommit()
   */
  public boolean getAutoCommit() throws SQLException {
    return con.getAutoCommit();
  }

  /**
   * @see java.sql.Connection#commit()
   */
  public void commit() throws SQLException {
    con.commit();
  }

  /**
   * @see java.sql.Connection#rollback()
   */
  public void rollback() throws SQLException {
    con.rollback();
  }

  /**
   * marks connection as closed in the pool
   * 
   * @see java.sql.Connection#close()
   */
  public void close() throws SQLException {
    checkedOn = System.currentTimeMillis();
    pool.freeConnection(this);
  }

  /**
   * @see java.sql.Connection#isClosed()
   */
  public boolean isClosed() throws SQLException {
    return con.isClosed();
  }

  /**
   * @see java.sql.Connection#getMetaData()
   */
  public DatabaseMetaData getMetaData() throws SQLException {
    return con.getMetaData();
  }

  /**
   * @see java.sql.Connection#setReadOnly(boolean)
   */
  public void setReadOnly(boolean readOnly) throws SQLException {
    con.setReadOnly(readOnly);
  }

  /**
   * @see java.sql.Connection#isReadOnly()
   */
  public boolean isReadOnly() throws SQLException {
    return con.isReadOnly();
  }

  /**
   * @see java.sql.Connection#setCatalog(java.lang.String)
   */
  public void setCatalog(String catalog) throws SQLException {
    con.setCatalog(catalog);
  }

  /**
   * @see java.sql.Connection#getCatalog()
   */
  public String getCatalog() throws SQLException {
    return con.getCatalog();
  }

  /**
   * @see java.sql.Connection#setTransactionIsolation(int)
   */
  public void setTransactionIsolation(int level) throws SQLException {
    con.setTransactionIsolation(level);
  }

  /**
   * @see java.sql.Connection#getTransactionIsolation()
   */
  public int getTransactionIsolation() throws SQLException {
    return con.getTransactionIsolation();
  }

  /**
   * @see java.sql.Connection#getWarnings()
   */
  public SQLWarning getWarnings() throws SQLException {
    return con.getWarnings();
  }

  /**
   * @see java.sql.Connection#clearWarnings()
   */
  public void clearWarnings() throws SQLException {
    con.clearWarnings();
  }

  /**
   * @see java.sql.Connection#createStatement(int, int)
   */
  public Statement createStatement(int resultSetType, int resultSetConcurrency) throws SQLException {
    return con.createStatement(resultSetType, resultSetConcurrency);
  }

  /**
   * @see java.sql.Connection#prepareStatement(java.lang.String, int, int)
   */
  public PreparedStatement prepareStatement(String sql, int resultSetType, int resultSetConcurrency)
      throws SQLException {
    return con.prepareStatement(sql, resultSetType, resultSetConcurrency);
  }

  /**
   * @see java.sql.Connection#prepareCall(java.lang.String, int, int)
   */
  public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency)
      throws SQLException {
    return con.prepareCall(sql, resultSetType, resultSetConcurrency);
  }

  /**
   * @see java.sql.Connection#getTypeMap()
   */
  public java.util.Map getTypeMap() throws SQLException {
    return con.getTypeMap();
  }

  /**
   * @see java.sql.Connection#setTypeMap(java.util.Map)
   */
  public void setTypeMap(java.util.Map map) throws SQLException {
    con.setTypeMap(map);
  }

  /**
   * @see java.sql.Connection#setHoldability(int)
   */
  public void setHoldability(int holdability) throws SQLException {
    con.setHoldability(holdability);
  }

  /**
   * @see java.sql.Connection#getHoldability()
   */
  public int getHoldability() throws SQLException {
    return con.getHoldability();
  }

  /**
   * @see java.sql.Connection#setSavepoint()
   */
  public Savepoint setSavepoint() throws SQLException {
    return con.setSavepoint();
  }

  /**
   * @see java.sql.Connection#setSavepoint(java.lang.String)
   */
  public Savepoint setSavepoint(String name) throws SQLException {
    return con.setSavepoint(name);
  }

  /**
   * @see java.sql.Connection#rollback(java.sql.Savepoint)
   */
  public void rollback(Savepoint savepoint) throws SQLException {
    con.rollback(savepoint);
  }

  /**
   * @see java.sql.Connection#releaseSavepoint(java.sql.Savepoint)
   */
  public void releaseSavepoint(Savepoint savepoint) throws SQLException {
    con.releaseSavepoint(savepoint);
  }

  /**
   * @see java.sql.Connection#createStatement(int, int, int)
   */
  public Statement createStatement(int resultSetType, int resultSetConcurrency,
      int resultSetHoldability) throws SQLException {
    return con.createStatement(resultSetType, resultSetConcurrency, resultSetHoldability);
  }

  /**
   * @see java.sql.Connection#prepareStatement(java.lang.String, int, int, int)
   */
  public PreparedStatement prepareStatement(String sql, int resultSetType,
      int resultSetConcurrency, int resultSetHoldability) throws SQLException {
    return con.prepareStatement(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
  }

  /**
   * @see java.sql.Connection#prepareCall(java.lang.String, int, int, int)
   */
  public CallableStatement prepareCall(String sql, int resultSetType, int resultSetConcurrency,
      int resultSetHoldability) throws SQLException {
    return con.prepareCall(sql, resultSetType, resultSetConcurrency, resultSetHoldability);
  }

  /**
   * @see java.sql.Connection#prepareStatement(java.lang.String, int)
   */
  public PreparedStatement prepareStatement(String sql, int autoGeneratedKeys) throws SQLException {
    return con.prepareStatement(sql, autoGeneratedKeys);
  }

  /**
   * @see java.sql.Connection#prepareStatement(java.lang.String, int[])
   */
  public PreparedStatement prepareStatement(String sql, int columnIndexes[]) throws SQLException {
    return con.prepareStatement(sql, columnIndexes);
  }

  /**
   * @see java.sql.Connection#prepareStatement(java.lang.String,
   *      java.lang.String[])
   */
  public PreparedStatement prepareStatement(String sql, String columnNames[]) throws SQLException {
    return con.prepareStatement(sql, columnNames);
  }

  /**
   * really closes the connection
   * 
   * @throws SQLException
   */
  public void cleanUpConnection() throws SQLException {
    con.close();
  }

  /**
   * Get the time stamp from last time the connection was checked
   * 
   * @return the time stamp
   */
  public long getLastCheckedStamp() {
    return checkedOn;
  }

  /**
   * Set the time stamp when the connection was last checked
   */
  public void setLastCheckedStamp(long checkedOn) {
    this.checkedOn = checkedOn;
  }

}
