/*
 * ClientTCPDAdapter.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.client.tcp;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.Socket;

import ro.raducora.connection.client.AdapterBase;
import ro.raducora.utils.encryption.key.StreamEncryptionKey;
import ro.raducora.utils.encryption.stream.DecryptedInputStream;
import ro.raducora.utils.encryption.stream.EncryptedOutputStream;
import ro.raducora.utils.io.Write;
import ro.raducora.utils.protocol.ConnectInfo;
import ro.raducora.utils.protocol.RedirectException;
import ro.raducora.utils.protocol.RemoteException;

/**
 * @author Radu Coravu<br>
 * 
 * Created on May 8, 2004 <br>
 * This is a client adapter which can connect directly to the server where the
 * database was found.This is used for the client driver.
 */
public class ClientTCPDAdapter implements AdapterBase {

  //TCP connection related items
  protected String remoteHost;

  protected int remotePort;

  private Socket connect = null;

  //Object Streams
  private ObjectOutputStream output = null;

  private ClientTCP clientTCP = null;

  //2 different keys
  //one for decripting input
  private StreamEncryptionKey inKey;

  //	one for encrypting output
  private StreamEncryptionKey outKey;

  private ObjectInputStream input = null;

  private boolean isConnected;

  /**
   * 
   * Constructor.
   * 
   * @param remoteHost
   *          Remote host to connect to
   * 
   * @param remotePort
   *          Port to connect to
   * 
   * @param inkey
   *          Decryption Key for input stream
   * @param outkey
   *          Encryption Key for output stream
   */
  public ClientTCPDAdapter(String remoteHost, int remotePort, StreamEncryptionKey inkey,
      StreamEncryptionKey outkey) {
    this.remoteHost = remoteHost;
    this.remotePort = remotePort;
    this.outKey = outkey;
    this.inKey = inkey;
    isConnected = false;
  }

  /**
   * Query Object
   * 
   * @return the queried object
   */
  public synchronized Object queryObject(Object sent) throws Exception {
    clientTCP.sendObject(sent);
    return clientTCP.getObject();
  }

  /**
   * @return true if connected
   */
  public boolean isConnected() {
    return isConnected;
  }

  /**
   * Connects to Remote Source
   * 
   * @param info
   *          -the connection info
   */
  public void connect(ConnectInfo info) throws Exception {
    if (isConnected == true) throw new Exception("This Client is already connected");
    Write.Line("Connecting to:" + remoteHost + " on port:" + remotePort);
    connect = new Socket(remoteHost, remotePort);
    connect.setKeepAlive(true);

    if (inKey == null || outKey == null) throw new NullPointerException("Null Encryption Keys");

    //initialize an input decryption key if not explicitly set
    inKey.setKey(connect.getInetAddress().getHostAddress() + ":" + connect.getPort());
    //initialize an output encryption key if not explicitly set
    outKey.setKey(connect.getLocalAddress().getHostAddress() + ":" + connect.getLocalPort());

    output = new ObjectOutputStream(new BufferedOutputStream(new EncryptedOutputStream(connect
        .getOutputStream(), outKey)));
    output.flush();

    input = new ObjectInputStream(new BufferedInputStream(new DecryptedInputStream(connect
        .getInputStream(), inKey)));

    clientTCP = new ClientTCP(input, output, true);

    Object ret = queryObject(info);

    if (ret instanceof Exception) {
      if (ret instanceof RedirectException) {
        Write.Line("Redirect Exception caught");
        try {
          disconnect();
        } catch (IOException exc) {
          //do nothing
        }
        RedirectException ex = (RedirectException) ret;
        remoteHost = ex.getRedirectHost();
        remotePort = ex.getRedirectPort();
        Write.Line("Redirecting to:" + remoteHost + " on port:" + remotePort);
        connect = new Socket(remoteHost, remotePort);
        connect.setKeepAlive(true);
        //initialize an input decryption key if not explicitly set
        inKey.setKey(connect.getInetAddress().getHostAddress() + ":" + connect.getPort());
        //initialize an output encryption key if not explicitly set
        outKey.setKey(connect.getLocalAddress().getHostAddress() + ":" + connect.getLocalPort());

        output = new ObjectOutputStream(new BufferedOutputStream(new EncryptedOutputStream(connect
            .getOutputStream(), outKey)));
        output.flush();

        input = new ObjectInputStream(new BufferedInputStream(new DecryptedInputStream(connect
            .getInputStream(), inKey)));

        clientTCP = new ClientTCP(input, output, true);

        ret = queryObject(info);

        if (ret instanceof Exception) { throw (Exception) ret; }

      } else {
        throw (RemoteException) ret;
      }
    }
    isConnected = true;
  }

  /**
   * Disconnect from remote source.
   */
  public void disconnect() throws IOException {
    isConnected = false;
    clientTCP.closeConnection();
    connect.close();
  }

  /**
   * Reloads the ClientTCPProps.
   *  
   */
  public void reloadProperties() throws Exception {
    return;
  }

  /**
   * Clone supported
   */
  public Object clone() throws CloneNotSupportedException {
    return super.clone();
  }

  /**
   * Return true
   */
  public boolean isAbstractConnection() {
    return true;
  }

  /**
   * returns null
   */
  public Object getConnectionStatus() {
    return null;
  }
}
