/*
 * ServerTCPAdapter.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */package ro.raducora.connection.server.tcp;

import java.io.BufferedInputStream;
import java.io.BufferedOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.net.ServerSocket;
import java.net.Socket;

import ro.raducora.connection.client.AdapterManager;
import ro.raducora.connection.server.Server;
import ro.raducora.utils.encryption.key.StreamEncryptionKey;
import ro.raducora.utils.encryption.stream.DecryptedInputStream;
import ro.raducora.utils.encryption.stream.EncryptedOutputStream;
import ro.raducora.utils.io.Write;
import ro.raducora.utils.props.ServerProps;

/**
 * @author Radu Coravu
 * 
 * <br>
 * A Server instance representing a DBAbstract Server
 */
public class ServerTCPAdapter extends Thread implements Server {

  private ServerSocket serverSocket = null;

  private ServerProps serverProps = null;

  private int PORT = 0;

  private boolean stop = false;

  private StreamEncryptionKey inKey;

  private StreamEncryptionKey outKey;

  private ObjectInputStream input;

  private ObjectOutputStream output;

  private boolean inSet = false;

  private boolean outSet = false;

  private AdapterManager adapterManager = null;

  /**
   * Intitialize Server Parameters
   * 
   * @param PORT
   *          the PORT to bind to
   * @param inkey
   *          key for decripting the input
   * @param outkey
   *          key for encripting the output
   * @throws IOException
   */
  public ServerTCPAdapter(ServerProps serverProps, AdapterManager adapterManager, int PORT,
      StreamEncryptionKey inkey, StreamEncryptionKey outkey) throws IOException {
    this.PORT = PORT;
    serverSocket = new ServerSocket(this.PORT);
    serverSocket.setSoTimeout(10000);
    this.inKey = inkey;
    this.outKey = outkey;
    inSet = this.inKey.isExplicitlySet();
    outSet = this.outKey.isExplicitlySet();
    this.serverProps = serverProps;
    this.adapterManager = adapterManager;
    this.setName("DBAbstract Server");
  }

  /**
   * Run the server thread
   */
  public void run() {
    while (!stop) {
      try {
        Socket socket = serverSocket.accept();

        socket.setKeepAlive(true);

        StreamEncryptionKey inClone = null;
        StreamEncryptionKey outClone = null;

        Write.Line("Connection Established");
        if (inKey == null || outKey == null) throw new NullPointerException();

        if (!inSet) {
          inClone = (StreamEncryptionKey) inKey.clone();
          //initialize an input decription key if not explicitly set
          inClone.setKey(socket.getInetAddress().getHostAddress() + ":" + socket.getPort());
        }
        if (!outSet) {
          outClone = (StreamEncryptionKey) outKey.clone();
          //initialize an output encription key if not explicitly
          // set
          outClone.setKey(socket.getLocalAddress().getHostAddress() + ":" + socket.getLocalPort());
        }

        output = new ObjectOutputStream(new BufferedOutputStream(new EncryptedOutputStream(socket
            .getOutputStream(), outClone)));
        output.flush();

        input = new ObjectInputStream(new BufferedInputStream(new DecryptedInputStream(socket
            .getInputStream(), inClone)));

        ServerThread.setMaxNoConnections(serverProps.getMaxNumberOfAllowedClients());
        Thread serverThread = new ServerThread(output, input, true,//serverProps.compressOutput(),
            serverProps.allowDirectConnection(), adapterManager);

        serverThread.setName(serverSocket.getInetAddress().getHostAddress() + ":"
            + serverSocket.getLocalPort());

        serverThread.start();

        input = null;
        output = null;
        inClone = null;
        outClone = null;
      } catch (IOException e) {
      } catch (Exception e) {
        Write.exception(e);
      }
    }
  }

  /**
   * Stop the Server
   */
  public void stopServer() {
    stop = true;
  }

  /**
   * Start the Server.
   */
  public synchronized void startServer() {
    stop = false;
    super.start();
  }

  /**
   * Reload properties
   */
  public void reload() throws Exception {
    serverProps.reload();
    adapterManager.reloadProperties();
  }

}
