/*
 * ServerThread.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.connection.server.tcp;

import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.sql.SQLException;
import java.util.zip.GZIPOutputStream;

import ro.raducora.connection.client.AdapterBase;
import ro.raducora.connection.client.AdapterManager;
import ro.raducora.utils.io.Write;
import ro.raducora.utils.protocol.ConnectInfo;
import ro.raducora.utils.protocol.RedirectException;
import ro.raducora.utils.protocol.RemoteException;

/**
 * @author Radu Coravu
 * 
 * <br>
 * A Server Thread serving one client
 */
public class ServerThread extends Thread {

  private static int numberOfConnections = 0;

  private static int maxNumberOfConnections = 100;

  private ObjectOutputStream output = null;

  private ObjectInputStream input = null;

  private boolean stop = false;

  private boolean gzipOutput = false;

  private boolean directConnection = false;

  private AdapterBase adapter = null;

  private AdapterManager adapterManager = null;

  /**
   * Elegantly stops the thread
   */
  public void stopThread() {
    this.stop = true;
  }

  /**
   * Sets the maximum number of accepted connections
   * 
   * @param maxNo
   *          the max no of allowed server threads
   */
  public static synchronized void setMaxNoConnections(int maxNo) {
    maxNumberOfConnections = maxNo;
  }

  /**
   * Constructor.
   * 
   * @param output
   *          the ObjectOutputStream
   * @param input
   *          the ObjectInputStream
   */
  public ServerThread(ObjectOutputStream output, ObjectInputStream input, boolean gzipOutput,
      boolean directConnection, AdapterManager adapterManager) {
    this.output = output;
    this.input = input;
    this.gzipOutput = gzipOutput;
    this.directConnection = directConnection;
    this.adapterManager = adapterManager;
    this.setDaemon(true);
    Write.Line("No of connections::" + (numberOfConnections + 1));
  }

  /**
   * Runs the server thread.
   */
  public void run() {
    numberOfConnections++;
    //for debug purpose
    this.setName(this.getName() + " - Thread " + numberOfConnections);
    //the thread name is actually a pair of ip:port
    if (numberOfConnections > maxNumberOfConnections) {
      closeConnection();
      return;
    }

    ConnectInfo info = null;
    adapter = null;

    try {
      info = (ConnectInfo) input.readObject();
    } catch (IOException e) {
      Write.exception(e);
      sendRemoteException(e);
      closeConnection();
      return;
    } catch (ClassNotFoundException e) {
      Write.exception(e);
      sendRemoteException(e);
      closeConnection();
      return;
    }

    if (info.checkLoop(this.getName().split("-")[0].trim())) {
      //loop present
      sendRemoteException(new IOException("Loop Detected"));
      disconnectAdapter();
      closeConnection();
      return;
    } else {
      //add current server id to path
      info.addToPath(this.getName().split("-")[0].trim());
    }

    adapter = adapterManager.getRightAdapter(info);

    if (adapter == null) {
      sendRemoteException(new IOException("No right adapter found"));
      closeConnection();
      return;
    }

    try {
      if (!adapter.isAbstractConnection()) {
        //found a real database
        if (directConnection && (info.getNoServersInPath() > 1)) {
          Write.Line("Redirecting connection::");
          String[] hp = this.getName().split("-")[0].trim().split(":");
          writeObject(new RedirectException(hp[0], Integer.parseInt(hp[1])));
          //disconnectAdapter();
          //closeConnection();
          //return;
        } else {
          //send OK to client, connection made
          //or any string as long as it is not an Exception :)
          writeObject("OK");
          Write.Line("OK to proceed::");
        }
      } else {
        //send OK to client, connection made
        //or any string as long as it is not an Exception :)
        writeObject(adapter.getConnectionStatus());
        Write.Line("OK to proceed::");
      }
    } catch (IOException e1) {
      sendRemoteException(e1);
      disconnectAdapter();
      closeConnection();
      return;
    }

    while (!stop) {
      try {
        writeObject(adapter.queryObject(input.readObject()));
      } catch (IOException e2) {
        disconnectAdapter();
        sendRemoteException(e2);
        closeConnection();
        return;
      } catch (ClassNotFoundException e2) {
        disconnectAdapter();
        sendRemoteException(e2);
        closeConnection();
        return;
      } catch (SQLException e) {
        Write.exception(e);
        disconnectAdapter();
        sendRemoteException(e);
      } catch (Exception e) {
        Write.exception(e);
        sendRemoteException(e);
        closeConnection();
        return;
      }
    }
  }

  private void disconnectAdapter() {
    try {
      adapter.disconnect();
    } catch (Exception e) {
      Write.exception(e);
    }
  }

  private void closeConnection() {
    Write.Line("Closing Connection Streams");
    numberOfConnections--;
    try {
      output.close();
    } catch (IOException e1) {
    }
    try {
      input.close();
    } catch (IOException e) {
    }

  }

  private void writeObject(Object obj) throws IOException {
    if (gzipOutput) {
      GZIPOutputStream outZip = new GZIPOutputStream(output);
      ObjectOutputStream gzipStream = new ObjectOutputStream(outZip);
      gzipStream.writeObject(obj);
      outZip.finish();
      gzipStream.flush();
    } else {
      output.writeObject(obj);
      output.flush();
    }
  }

  private void sendRemoteException(Exception e) {
    try {
      writeObject(new RemoteException(e));
    } catch (IOException e1) {
      Write.exception(e);
    }
  }

}
