/**
 * Modified on 04.12.2003
 *			by Radu Coravu
 */
package ro.raducora.utils.encryption;

import java.security.InvalidKeyException;

/**
 * This class implements the RC4 (TM) stream cipher.
 */
public final class RC4 // must be final for security reasons
{

  //state true=encrypt
  //    		false=decrypt
  private boolean encript = false;

  /**
   * @return true for encrypt false for decrypt
   */
  public boolean getState() {
    return encript;
  }

  // RC4 constants and variables
  //............................................................................

  /**
   * Will hold the contents of the current set S-box.
   */
  private int[] sBox = new int[256];

  /**
   * The two indices for the S-box computation referred to as i and j in
   * Schneier.
   */
  private int x, y;

  /**
   * The block size of this cipher. Being a stream cipher this value is 1!
   */
  private static final int BLOCK_SIZE = 1;

  // Constructor, finalizer, and clone()
  //............................................................................

  /**
   * Constructs an RC4 cipher object, in the UNINITIALIZED state.
   */
  public RC4() {
  }

  /**
   * Always throws a CloneNotSupportedException (cloning of ciphers is not
   * supported for security reasons).
   */
  public final Object clone() throws CloneNotSupportedException {
    throw new CloneNotSupportedException();
  }

  // Implementation of JCE methods
  //............................................................................

  /**
   * <b>SPI </b>: Returns the length of an input block, in bytes.
   * 
   * @return the length in bytes of an input block for this cipher.
   */
  public int engineBlockSize() {
    return BLOCK_SIZE;
  }

  /**
   * <b>SPI </b>: Initializes this cipher for encryption, using the specified
   * key.
   * 
   * @param key
   *          the key to use for encryption.
   * @exception InvalidKeyException
   *              if the key is invalid.
   */
  public void engineInitEncrypt(byte[] key) throws InvalidKeyException {
    makeKey(key);
  }

  /**
   * <b>SPI </b>: Initializes this cipher for decryption, using the specified
   * key.
   * 
   * @param key
   *          the key to use for decryption.
   * @exception InvalidKeyException
   *              if the key is invalid.
   */
  public void engineInitDecrypt(byte[] key) throws InvalidKeyException {
    makeKey(key);
  }

  /**
   * <b>SPI </b>: This is the main engine method for updating data.
   * <p>
   * <i>in </i> and <i>out </i> may be the same array, and the input and output
   * regions may overlap.
   * 
   * @param in
   *          the input data.
   * @param inOffset
   *          the offset into in specifying where the data starts.
   * @param inLen
   *          the length of the subarray.
   * @param out
   *          the output array.
   * @param outOffset
   *          the offset indicating where to start writing into the out array.
   * @return the number of bytes written.
   */
  protected int engineUpdate(byte[] in, int inOffset, int inLen, byte[] out, int outOffset) {
    if (inLen < 0) throw new IllegalArgumentException("inLen < 0");

    // Avoid overlapping input and output regions.
    if (in == out
        && (outOffset >= inOffset && outOffset < (long) inOffset + inLen || inOffset >= outOffset
            && inOffset < (long) outOffset + inLen)) {
      byte[] newin = new byte[inLen];
      System.arraycopy(in, inOffset, newin, 0, inLen);
      in = newin;
      inOffset = 0;
    }
    rc4(in, inOffset, inLen, out, outOffset);
    return inLen;
  }

  // Own methods
  //............................................................................

  /**
   * RC4 encryption/decryption. The input and output regions are assumed not to
   * overlap.
   * 
   * @param in
   *          the input data.
   * @param inOffset
   *          the offset into in specifying where the data starts.
   * @param inLen
   *          the length of the subarray.
   * @param out
   *          the output array.
   * @param outOffset
   *          the offset indicating where to start writing into the out array.
   */
  public void rc4(byte[] in, int inOffset, int inLen, byte[] out, int outOffset) {
    int xorIndex, t;

    for (int i = 0; i < inLen; i++) {
      x = (x + 1) & 0xFF;
      y = (sBox[x] + y) & 0xFF;

      t = sBox[x];
      sBox[x] = sBox[y];
      sBox[y] = t;

      xorIndex = (sBox[x] + sBox[y]) & 0xFF;
      out[outOffset++] = (byte) (in[inOffset++] ^ sBox[xorIndex]);
    }
  }

  /**
   * Expands a user-key to a working key schedule.
   * <p>
   * The key bytes are first extracted from the user-key and then used to build
   * the contents of this key schedule.
   * <p>
   * The method's only exceptions are when the user-key's contents are null, or
   * a byte array of zero length.
   * 
   * @param key
   *          the user-key object to use.
   * @exception InvalidKeyException
   *              if one of the following occurs:
   *              <ul>
   *              <li>key.getEncoded() == null;
   *              <li>The encoded byte array form of the key is zero-length;
   *              </ul>
   */
  private void makeKey(byte[] key) throws InvalidKeyException {

    byte[] userkey = key;
    if (userkey == null) throw new InvalidKeyException("RC4: Null user key");

    int len = userkey.length;
    if (len == 0) throw new InvalidKeyException("RC4: Invalid user key length");

    x = y = 0;
    for (int i = 0; i < 256; i++)
      sBox[i] = i;

    int i1 = 0, i2 = 0, t;

    for (int i = 0; i < 256; i++) {
      i2 = ((userkey[i1] & 0xFF) + sBox[i] + i2) & 0xFF;

      t = sBox[i];
      sBox[i] = sBox[i2];
      sBox[i2] = t;

      i1 = (i1 + 1) % len;
    }
  }
}
