/*
 * ClientDBProperties.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.props.properties;

import java.io.BufferedInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.MissingResourceException;
import java.util.PropertyResourceBundle;

import ro.raducora.utils.props.ClientDBProps;
import ro.raducora.utils.props.Resolver;

/**
 * @author Radu Coravu
 *  
 */

public class ClientDBProperties implements ClientDBProps {

  private String propertiesFilename = "./conf/ClientDB.properties";

  private static String DEFAULT_DB_SERVER_HOSTNAME = null; // "localhost";

  private static String DEFAULT_DB_SERVER_SUBPROTOCOL = null; // "mysql";

  private static String DEFAULT_DRIVER_CLASS_NAME = null; //"org.gjt.mm.mysql.Driver";

  private static String DEFAULT_DRIVER_JAR_PATH = null; //"/lib/";

  private static String DEFAULT_DRIVER_JAR_NAME = null; //"mm.mysql-2.0.14.jar";

  private static int DEFAULT_MIN_NO_CONN = 0;

  private static int DEFAULT_MAX_NO_CONN = 0;

  private PropertyResourceBundle bundle = null;

  /**
   * Uses default properties filename path <code>"./conf/ClientDB.properties"</code>
   */
  public ClientDBProperties() throws FileNotFoundException, IOException {
    reload();
  }

  /**
   * @param filename
   *          <code>properties filename</code>
   */
  public ClientDBProperties(String filename) throws FileNotFoundException, IOException {
    propertiesFilename = filename;
    reload();
  }

  /**
   * Get Real DB Name with aid of key.
   */
  public String getRealDBName(String dbname) {
    return dbname;
  }

  /**
   * Gets the Server Hostname
   */
  public String getServerHostname(String dbname) {
    String ret = null;
    try {
      ret = bundle.getString(dbname + ".DB_SERVER_HOSTNAME");
    } catch (MissingResourceException ex) {
      return DEFAULT_DB_SERVER_HOSTNAME;
    }
    return ret;
  }

  /**
   * Gets the Driver Class Name
   */
  public String getDriverClassName(String dbname) {
    String ret = null;
    try {
      ret = bundle.getString(dbname + ".DRIVER_CLASS_NAME");
    } catch (MissingResourceException ex) {
      return DEFAULT_DRIVER_CLASS_NAME;
    }
    return ret;
  }

  /**
   * Gets Driver Jar path
   */
  public String getDriverJarPath(String dbname) {
    String ret = null;
    try {
      ret = bundle.getString(dbname + ".DRIVER_JAR_PATH");
    } catch (MissingResourceException ex) {
      return DEFAULT_DRIVER_JAR_PATH;
    }
    return ret;
  }

  /**
   * Get jar filename
   */
  public String getDriverJarFilename(String dbname) {
    String ret = null;
    try {
      ret = bundle.getString(dbname + ".DRIVER_JAR_NAME");
    } catch (MissingResourceException ex) {
      return DEFAULT_DRIVER_JAR_NAME;
    }
    return ret;
  }

  /**
   * Get Server Subprotocol.
   */
  public String getServerSubprotocol(String dbname) {
    String ret = null;
    try {
      ret = bundle.getString(dbname + ".DB_SERVER_SUBPROTOCOL");
    } catch (MissingResourceException ex) {
      return DEFAULT_DB_SERVER_SUBPROTOCOL;
    }
    return ret;
  }

  /**
   * Get min no of connections for dbname.
   */
  public int getMinNoConn(String dbname) {
    int ret = 0;
    try {
      ret = Integer.parseInt(bundle.getString(dbname + ".MIN_NO_CONN"));
    } catch (MissingResourceException ex) {
      return DEFAULT_MIN_NO_CONN;
    }
    return ret;
  }

  /**
   * Get min no of connections for dbname.
   */
  public int getMaxNoConn(String dbname) {
    int ret = 0;
    try {
      ret = Integer.parseInt(bundle.getString(dbname + ".MAX_NO_CONN"));
    } catch (MissingResourceException ex) {
      return DEFAULT_MAX_NO_CONN;
    }
    return ret;
  }

  /**
   * Reloads Properties from file
   */
  public void reload() throws FileNotFoundException, IOException {
    bundle = new PropertyResourceBundle(new BufferedInputStream(Resolver
        .solveSource(propertiesFilename)));
    try {
      DEFAULT_DB_SERVER_HOSTNAME = bundle.getString("DEFAULT_DB_SERVER_HOSTNAME");
    } catch (MissingResourceException ex) {
    }
    try {
      DEFAULT_DB_SERVER_SUBPROTOCOL = bundle.getString("DEFAULT_DB_SERVER_SUBPROTOCOL");
    } catch (MissingResourceException ex) {
    }
    try {
      DEFAULT_DRIVER_CLASS_NAME = bundle.getString("DEFAULT_DRIVER_CLASS_NAME");
    } catch (MissingResourceException ex) {
    }
    try {
      DEFAULT_DRIVER_JAR_PATH = bundle.getString("DEFAULT_DRIVER_JAR_PATH");
    } catch (MissingResourceException ex) {
    }
    try {
      DEFAULT_DRIVER_JAR_NAME = bundle.getString("DEFAULT_DRIVER_JAR_NAME");

    } catch (MissingResourceException ex) {
    }
    try {
      DEFAULT_MIN_NO_CONN = Integer.parseInt(bundle.getString("DEFAULT_MIN_NO_CONN"));
    } catch (MissingResourceException ex) {
    }

    try {
      DEFAULT_MAX_NO_CONN = Integer.parseInt(bundle.getString("DEFAULT_MAX_NO_CONN"));
    } catch (MissingResourceException ex) {
    }
  }

  //    public static void main(String[] args) {
  //        ClientDBProperties props = new ClientDBProperties();
  //        System.out.println("Hostname:" + props.getServerHostname("mini"));
  //        System.out.println("Server Subprotocol:" +
  // props.getServerSubprotocol("mini"));
  //        System.out.println("Path to Jar:" + props.getDriverJarPath("mini"));
  //        System.out.println("Jar Filename:" +
  // props.getDriverJarFilename("mini"));
  //        System.out.println("Driver class name:" +
  // props.getDriverClassName("mini"));
  //    }
}
