/*
 * SerializedResultSet.java
 * 
 * Copyright 2004 Coravu Publications All Rights Reserved.
 * 
 * This file is part of the DBAbstract Project.
 * 
 * The DBAbstract Project is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or (at your
 * option) any later version.
 * 
 * The DBAbstract Project is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General
 * Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * the DBAbstract Project; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * http://www.raducora.ro
 * 
 * CONTACT: email = radu_coravu@sync.ro
 */
package ro.raducora.utils.protocol;

import java.io.InputStream;
import java.io.Reader;
import java.io.Serializable;
import java.math.BigDecimal;
import java.net.URL;
import java.sql.Array;
import java.sql.Blob;
import java.sql.Clob;
import java.sql.Date;
import java.sql.Ref;
import java.sql.ResultSet;
import java.sql.ResultSetMetaData;
import java.sql.SQLException;
import java.sql.SQLWarning;
import java.sql.Statement;
import java.sql.Time;
import java.sql.Timestamp;
import java.util.Calendar;
import java.util.Map;

import ro.raducora.utils.io.Write;

/**
 * A duplicate of java.sql.ResultSet which is serializable
 */
public class SerializedResultSet implements Serializable, ResultSet {

  private java.util.LinkedList lstColumnData = new java.util.LinkedList();

  private int currentRecord = -1;

  private int totalRecords = 0;

  private ResultSetMetaData metaData = null;

  /**
   * Takes as a parameter the ResultSet to wrap
   * 
   * @param rs
   *          the ResultSet to read into it's serializable list
   * @throws SQLException
   */
  public SerializedResultSet(ResultSet rs) throws SQLException {
    currentRecord = -1;
    totalRecords = 0;

    ResultSetMetaData rsMetaData = rs.getMetaData();
    metaData = new SerializedResultSetMetaData(rs.getMetaData());

    for (int i = 1; i <= rsMetaData.getColumnCount(); i++) {
      this.addColumn(i);
    }

    while (rs.next()) {
      for (int column = 1; column <= rsMetaData.getColumnCount(); column++) {
        this.addColumnData(column, rs.getObject(column));
      }
    }
  }

  /**
   * Moves the cursor down one row from its current position in the result set.
   * Initially the cursor is at the top of the first row.
   * 
   * @return <code>true</code> if the new current row is valid; <code>false</code>
   *         otherwise.
   */
  public boolean next() throws SQLException {
    int totalRecords = recordCount();
    if (totalRecords == 0) {
      currentRecord = 0;
      return false;
    }
    if (currentRecord == -1) {
      currentRecord = 0;
      return true;
    }
    if (currentRecord < totalRecords && currentRecord >= 0 && currentRecord != (totalRecords - 1)) {
      currentRecord++;
      return true;
    }
    return false;
  }

  /**
   * Returns the total number of records in the resultset.
   * 
   * @return <code>integer</code> the total number of records in the result
   *         set.
   */
  public int recordCount() {
    try {
      java.util.LinkedList columnData = (java.util.LinkedList) lstColumnData.get(0);
      return columnData.size();
    } catch (Exception ex) {
      return 0;
    }
  }

  /**
   * Positions the cursor at the first value of the result set.
   * 
   * @return <code>true</code> if the first row is valid; <code>false</code>
   *         otherwise.
   */
  public boolean first() {
    currentRecord = -1;
    return true;
  }

  /**
   * Positions the cursor at the last value of the table result set.
   * 
   * @return <code>true</code> if the first row is valid; <code>false</code>
   *         otherwise.
   */
  public boolean last() {
    currentRecord = recordCount() - 1;
    return true;
  }

  /**
   * Get the field value specifed by the column name and the row.
   * 
   * @param columnName
   *          The name of the table column.
   * @param row
   *          The row number int the column.
   * @return An <code>Object</code> containing the data.
   * @throws SQLException
   *           if the row number is greater than the size of the column data or
   *           given the column name is invalid.
   */
  private Object getField(String columnName, int row) throws SQLException {
    if (row == -1) {
      currentRecord++;
      row = currentRecord;
    }
    int iColumnIndex = searchItem(columnName.toUpperCase());
    java.util.LinkedList columnData = ((java.util.LinkedList) lstColumnData.get(iColumnIndex - 1));
    return columnData.get(row);
  }

  private int searchItem(String columnName) throws SQLException {
    for (int i = 1; i <= metaData.getColumnCount(); i++) {
      if (metaData.getColumnName(i).equalsIgnoreCase(columnName)) return i;
    }
    return -1;
  }

  /**
   * Get the field value specifed by the column name and the row.
   * 
   * @param columnIndex
   *          The name of the table column.
   * @param row
   *          The row number int the column.
   * @return An <code>Object</code> containing the data.
   * @throws SQLException
   *           if the row number is greater than the size of the column data or
   *           given the column name is invalid.
   */
  private Object getField(int columnIndex, int row) throws SQLException {
    if (row == -1) {
      currentRecord++;
      row = currentRecord;
    }
    int iColumnIndex = columnIndex;
    java.util.LinkedList columnData = ((java.util.LinkedList) lstColumnData.get(iColumnIndex - 1));
    return columnData.get(row);
  }

  /**
   * Get a String value specified by the column name.
   * 
   * @param columnName
   *          The name of the table column.
   * @return A <code>String</code> containing the data.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public String getString(String columnName) throws SQLException {
    Object ret = this.getField(columnName, currentRecord);
    if (ret == null) { return null; }
    return ret.toString();
  }

  /**
   * Get Date specified by the column name.
   * 
   * @param columnName
   *          The name of the table column.
   * @return A <code>String</code> containing the date.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public Date getDate(String columnName) throws SQLException {
    String dateString = this.getString(columnName);
    if (dateString != null) {
      return java.sql.Date.valueOf(dateString);
    } else {
      return null;
    }
  }

  /**
   * Get Time specified by the column name.
   * 
   * @param columnName
   *          The name of the table column.
   * @return A <code>String</code> containing the time.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public Time getTime(String columnName) throws SQLException {
    String dateTime = this.getString(columnName);
    if (dateTime != null) {
      return Time.valueOf(dateTime);
    } else {
      return null;
    }
  }

  /**
   * Get an Object specified by the column name.
   * 
   * @param columnName
   *          The name of the table column.
   * @return An <code>Object</code> containing the data.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public Object getObject(String columnName) throws SQLException {
    return this.getField(columnName, currentRecord);
  }

  /**
   * Get a long value specified by the column name.
   * 
   * @param columnName
   *          The name of the table column.
   * @return A <code>long</code> containing the data.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public long getLong(String columnName) throws SQLException {
    Object obj = this.getField(columnName, currentRecord);
    if (obj == null) {
      return 0;
    } else {
      try {
        return ((java.math.BigDecimal) obj).longValue();
      } catch (Throwable ex) {
        try {
          return ((Long) obj).longValue();
        } catch (Throwable ex1) {
          try {
            return ((Integer) obj).longValue();
          } catch (Throwable ex2) {
            return Long.parseLong((obj.toString()).trim());
          }
        }
      }
    }
  }

  /**
   * Get a double value specified by the column name.
   * 
   * @param columnName
   *          The name of the table column.
   * @return A <code>double</code> containing the data.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public double getDouble(String columnName) throws SQLException {
    Object obj = this.getField(columnName, currentRecord);
    if (obj == null) {
      return 0;
    } else {
      try {
        return ((java.math.BigDecimal) obj).doubleValue();
      } catch (Throwable ex) {
        try {
          double ret = ((Double) obj).doubleValue();
          return ret;
        } catch (Throwable e) {
          return Double.parseDouble((obj.toString()).trim());
        }
      }
    }
  }

  /**
   * Get a java.sql.Timestamp Object specified by the column name.
   * 
   * @param columnName
   *          The name of the table column.
   * @return An <code>java.sql.Timestamp Object</code> containing the data.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public java.sql.Timestamp getTimestamp(String columnName) throws SQLException {
    Object obj = this.getField(columnName, currentRecord);
    if (obj == null) {
      return null;
    } else {
      try {
        return ((java.sql.Timestamp) obj);
      } catch (Exception e) {
        return Timestamp.valueOf(obj.toString());
      }
    }
  }

  /**
   * Get a integer value specified by the column name.
   * 
   * @param columnName
   *          The name of the table column.
   * @return An <code>integer</code> containing the data.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public int getInt(String columnName) throws SQLException {
    Object obj = this.getField(columnName, currentRecord);
    if (obj == null) {
      return 0;
    } else {
      try {
        return ((Integer) obj).intValue();
      } catch (Throwable ex) {
        try {
          return ((java.math.BigDecimal) obj).intValue();
        } catch (Throwable ex1) {
          try {
            return ((Long) obj).intValue();
          } catch (Throwable ex2) {
            return Integer.parseInt((obj.toString()).trim());
          }
        }
      }
    }

  }

  /**
   * Adds column data to the list.
   * 
   * @param ColumnNumber
   *          The index of the column.
   * @param newValue
   *          The new value to be added.
   */
  private void addColumnData(int ColumnNumber, Object newValue) {
    ((java.util.LinkedList) lstColumnData.get(ColumnNumber - 1)).addLast(newValue);
  }

  /**
   * Adds column name to the list.
   * 
   * @param columnSequence
   *          The index of the column.
   */
  private void addColumn(int columnSequence) {
    java.util.LinkedList linkData = new java.util.LinkedList();
    lstColumnData.addLast(linkData);
    ((java.util.LinkedList) lstColumnData.get(columnSequence - 1)).clear();
  }

  /**
   * @see java.sql.ResultSet#close()
   */
  public void close() throws SQLException {
    return;
  }

  /**
   *  method not supported
   * 
   * @see java.sql.ResultSet#wasNull()
   */
  public boolean wasNull() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * @see java.sql.ResultSet#getString(int)
   */
  public String getString(int columnIndex) throws SQLException {
    Object ret = getField(columnIndex - 1, currentRecord);
    if (ret == null) { return null; }
    return ret.toString();
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getBoolean(int)
   */
  public boolean getBoolean(int columnIndex) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getByte(int)
   */
  public byte getByte(int columnIndex) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getShort(int)
   */
  public short getShort(int columnIndex) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * Get an integer value specified by the column name.
   * 
   * @param columnIndex
   *          The name of the table column.
   * @return An <code>integer</code> containing the data.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public int getInt(int columnIndex) throws SQLException {
    Object obj = this.getField(columnIndex, currentRecord);
    if (obj == null) {
      return 0;
    } else {
      try {
        return ((java.math.BigDecimal) obj).intValue();
      } catch (Throwable exe) {
        return Integer.parseInt(((String) obj).trim());
      }
    }
  }

  /**
   * Get a long value specified by the column name.
   * 
   * @param columnIndex
   *          The name of the table column.
   * @return A <code>long</code> containing the data.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public long getLong(int columnIndex) throws SQLException {
    Object obj = this.getField(columnIndex - 1, currentRecord);
    if (obj == null) {
      return 0;
    } else {
      try {
        return ((java.math.BigDecimal) obj).longValue();
      } catch (Throwable ex) {
        try {
          return ((Long) obj).longValue();
        } catch (Throwable ex1) {
          try {
            return ((Integer) obj).longValue();
          } catch (Throwable ex2) {
            return Long.parseLong(((String) obj).trim());
          }
        }
      }
    }
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getFloat(int)
   */
  public float getFloat(int columnIndex) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * Get a double value specified by the column name.
   * 
   * @param columnIndex
   *          The name of the table column.
   * @return A <code>double</code> containing the data.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public double getDouble(int columnIndex) throws SQLException {
    Object obj = this.getField(columnIndex - 1, currentRecord);
    if (obj == null) {
      return 0;
    } else {
      try {
        return ((java.math.BigDecimal) obj).doubleValue();
      } catch (Throwable ex) {
        try {
          double ret = ((Double) obj).doubleValue();
          return ret;
        } catch (Throwable e) {
          return Double.parseDouble(((String) obj).trim());
        }
      }
    }
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getBigDecimal(int, int)
   */
  public BigDecimal getBigDecimal(int columnIndex, int scale) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getBytes(int)
   */
  public byte[] getBytes(int columnIndex) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * Get Date specified by the column name.
   * 
   * @param columnIndex
   *          The name of the table column.
   * @return A <code>String</code> containing the date.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public Date getDate(int columnIndex) throws SQLException {
    String dateString = this.getString(columnIndex);
    if (dateString != null) {
      return java.sql.Date.valueOf(dateString);
    } else {
      return null;
    }
  }

  /**
   * Get Time specified by the column name.
   * 
   * @param columnIndex
   *          The name of the table column.
   * @return A <code>String</code> containing the time.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public Time getTime(int columnIndex) throws SQLException {
    String dateTime = this.getString(columnIndex);
    if (dateTime != null) {
      return Time.valueOf(dateTime);
    } else {
      return null;
    }
  }

  /**
   * Get a java.sql.Timestamp Object specified by the column name.
   * 
   * @param columnIndex
   *          The name of the table column.
   * @return An <code>java.sql.Timestamp Object</code> containing the data.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public java.sql.Timestamp getTimestamp(int columnIndex) throws SQLException {
    Object obj = this.getField(columnIndex - 1, currentRecord);
    if (obj == null) {
      return null;
    } else {
      return ((java.sql.Timestamp) obj);
    }
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getAsciiStream(int)
   */
  public InputStream getAsciiStream(int columnIndex) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getUnicodeStream(int)
   */
  public InputStream getUnicodeStream(int columnIndex) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getBinaryStream(int)
   */
  public InputStream getBinaryStream(int columnIndex) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getBoolean(java.lang.String)
   */
  public boolean getBoolean(String columnName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getByte(java.lang.String)
   */
  public byte getByte(String columnName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getShort(java.lang.String)
   */
  public short getShort(String columnName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getFloat(java.lang.String)
   */
  public float getFloat(String columnName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getBigDecimal(java.lang.String, int)
   */
  public BigDecimal getBigDecimal(String columnName, int scale) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getBytes(java.lang.String)
   */
  public byte[] getBytes(String columnName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getAsciiStream(java.lang.String)
   */
  public InputStream getAsciiStream(String columnName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getUnicodeStream(java.lang.String)
   */
  public InputStream getUnicodeStream(String columnName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getBinaryStream(java.lang.String)
   */
  public InputStream getBinaryStream(String columnName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getWarnings()
   */
  public SQLWarning getWarnings() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#clearWarnings()
   */
  public void clearWarnings() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getCursorName()
   */
  public String getCursorName() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * @see java.sql.ResultSet#getMetaData()
   */
  public ResultSetMetaData getMetaData() throws SQLException {
    return metaData;
  }

  /**
   * Get an Object specified by the column name.
   * 
   * @param columnIndex
   *          The name of the table column.
   * @return An <code>Object</code> containing the data.
   * @throws SQLException
   *           if an invalid column Name is given.
   */
  public Object getObject(int columnIndex) throws SQLException {
    return this.getField(columnIndex, currentRecord);
  }

  /**
   * @see java.sql.ResultSet#findColumn(java.lang.String)
   */
  public int findColumn(String columnName) throws SQLException {
    return searchItem(columnName);
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getCharacterStream(int)
   */
  public Reader getCharacterStream(int columnIndex) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getCharacterStream(java.lang.String)
   */
  public Reader getCharacterStream(String columnName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getBigDecimal(int)
   */
  public BigDecimal getBigDecimal(int columnIndex) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getBigDecimal(java.lang.String)
   */
  public BigDecimal getBigDecimal(String columnName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * @see java.sql.ResultSet#isBeforeFirst()
   */
  public boolean isBeforeFirst() throws SQLException {
    return (currentRecord == -1);
  }

  /**
   * @see java.sql.ResultSet#isAfterLast()
   */
  public boolean isAfterLast() throws SQLException {
    return isBeforeFirst();
  }

  /**
   * @see java.sql.ResultSet#isFirst()
   */
  public boolean isFirst() throws SQLException {
    return (currentRecord == 0);
  }

  /**
   * @see java.sql.ResultSet#isLast()
   */
  public boolean isLast() throws SQLException {
    return (currentRecord == recordCount() - 1);
  }

  /**
   * @see java.sql.ResultSet#beforeFirst()
   */
  public void beforeFirst() throws SQLException {
    currentRecord = -1;
  }

  /**
   * @see java.sql.ResultSet#afterLast()
   */
  public void afterLast() throws SQLException {
    currentRecord = recordCount();
  }

  /**
   * @see java.sql.ResultSet#getRow()
   */
  public int getRow() throws SQLException {
    return currentRecord;
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#absolute(int)
   */
  public boolean absolute(int row) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#relative(int)
   */
  public boolean relative(int rows) throws SQLException {

    throw new SQLException("Method not supported");
  }

  /**
   * @see java.sql.ResultSet#previous()
   */
  public boolean previous() throws SQLException {
    int totalRecords = recordCount();
    if (totalRecords == 0) {
      currentRecord = 0;
      return false;
    }
    if (currentRecord >= 0) {
      currentRecord--;
      return true;
    }
    return false;
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#setFetchDirection(int)
   */
  public void setFetchDirection(int direction) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getFetchDirection()
   */
  public int getFetchDirection() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#setFetchSize(int)
   */
  public void setFetchSize(int rows) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getFetchSize()
   */
  public int getFetchSize() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * @see java.sql.ResultSet#getType()
   */
  public int getType() throws SQLException {
    return TYPE_FORWARD_ONLY;
  }

  /**
   * @see java.sql.ResultSet#getConcurrency()
   */
  public int getConcurrency() throws SQLException {
    return CONCUR_READ_ONLY;
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#rowUpdated()
   */
  public boolean rowUpdated() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#rowInserted()
   */
  public boolean rowInserted() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#rowDeleted()
   */
  public boolean rowDeleted() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateNull(int)
   */
  public void updateNull(int columnIndex) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateBoolean(int, boolean)
   */
  public void updateBoolean(int columnIndex, boolean x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateByte(int, byte)
   */
  public void updateByte(int columnIndex, byte x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateShort(int, short)
   */
  public void updateShort(int columnIndex, short x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateInt(int, int)
   */
  public void updateInt(int columnIndex, int x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateLong(int, long)
   */
  public void updateLong(int columnIndex, long x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateFloat(int, float)
   */
  public void updateFloat(int columnIndex, float x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateDouble(int, double)
   */
  public void updateDouble(int columnIndex, double x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateBigDecimal(int, java.math.BigDecimal)
   */
  public void updateBigDecimal(int columnIndex, BigDecimal x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateString(int, java.lang.String)
   */
  public void updateString(int columnIndex, String x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateBytes(int, byte[])
   */
  public void updateBytes(int columnIndex, byte[] x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateDate(int, java.sql.Date)
   */
  public void updateDate(int columnIndex, Date x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateTime(int, java.sql.Time)
   */
  public void updateTime(int columnIndex, Time x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateTimestamp(int, java.sql.Timestamp)
   */
  public void updateTimestamp(int columnIndex, Timestamp x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateAsciiStream(int, java.io.InputStream, int)
   */
  public void updateAsciiStream(int columnIndex, InputStream x, int length) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateBinaryStream(int, java.io.InputStream, int)
   */
  public void updateBinaryStream(int columnIndex, InputStream x, int length) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateCharacterStream(int, java.io.Reader, int)
   */
  public void updateCharacterStream(int columnIndex, Reader x, int length) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateObject(int, java.lang.Object, int)
   */
  public void updateObject(int columnIndex, Object x, int scale) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateObject(int, java.lang.Object)
   */
  public void updateObject(int columnIndex, Object x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateNull(java.lang.String)
   */
  public void updateNull(String columnName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateBoolean(java.lang.String, boolean)
   */
  public void updateBoolean(String columnName, boolean x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateByte(java.lang.String, byte)
   */
  public void updateByte(String columnName, byte x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateShort(java.lang.String, short)
   */
  public void updateShort(String columnName, short x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateInt(java.lang.String, int)
   */
  public void updateInt(String columnName, int x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateLong(java.lang.String, long)
   */
  public void updateLong(String columnName, long x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateFloat(java.lang.String, float)
   */
  public void updateFloat(String columnName, float x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateDouble(java.lang.String, double)
   */
  public void updateDouble(String columnName, double x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateBigDecimal(java.lang.String,
   *      java.math.BigDecimal)
   */
  public void updateBigDecimal(String columnName, BigDecimal x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateString(java.lang.String, java.lang.String)
   */
  public void updateString(String columnName, String x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateBytes(java.lang.String, byte[])
   */
  public void updateBytes(String columnName, byte[] x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateDate(java.lang.String, java.sql.Date)
   */
  public void updateDate(String columnName, Date x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateTime(java.lang.String, java.sql.Time)
   */
  public void updateTime(String columnName, Time x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateTimestamp(java.lang.String,
   *      java.sql.Timestamp)
   */
  public void updateTimestamp(String columnName, Timestamp x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateAsciiStream(java.lang.String,
   *      java.io.InputStream, int)
   */
  public void updateAsciiStream(String columnName, InputStream x, int length) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateBinaryStream(java.lang.String,
   *      java.io.InputStream, int)
   */
  public void updateBinaryStream(String columnName, InputStream x, int length) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateCharacterStream(java.lang.String,
   *      java.io.Reader, int)
   */
  public void updateCharacterStream(String columnName, Reader reader, int length)
      throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateObject(java.lang.String, java.lang.Object,
   *      int)
   */
  public void updateObject(String columnName, Object x, int scale) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateObject(java.lang.String, java.lang.Object)
   */
  public void updateObject(String columnName, Object x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#insertRow()
   */
  public void insertRow() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateRow()
   */
  public void updateRow() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#deleteRow()
   */
  public void deleteRow() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#refreshRow()
   */
  public void refreshRow() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#cancelRowUpdates()
   */
  public void cancelRowUpdates() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#moveToInsertRow()
   */
  public void moveToInsertRow() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#moveToCurrentRow()
   */
  public void moveToCurrentRow() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getStatement()
   */
  public Statement getStatement() throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getObject(int, java.util.Map)
   */
  public Object getObject(int i, Map map) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getRef(int)
   */
  public Ref getRef(int i) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getBlob(int)
   */
  public Blob getBlob(int i) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getClob(int)
   */
  public Clob getClob(int i) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getArray(int)
   */
  public Array getArray(int i) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getObject(java.lang.String, java.util.Map)
   */
  public Object getObject(String colName, Map map) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getRef(java.lang.String)
   */
  public Ref getRef(String colName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getBlob(java.lang.String)
   */
  public Blob getBlob(String colName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getClob(java.lang.String)
   */
  public Clob getClob(String colName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getArray(java.lang.String)
   */
  public Array getArray(String colName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getDate(int, java.util.Calendar)
   */
  public Date getDate(int columnIndex, Calendar cal) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getDate(java.lang.String, java.util.Calendar)
   */
  public Date getDate(String columnName, Calendar cal) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getTime(int, java.util.Calendar)
   */
  public Time getTime(int columnIndex, Calendar cal) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getTime(java.lang.String, java.util.Calendar)
   */
  public Time getTime(String columnName, Calendar cal) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getTimestamp(int, java.util.Calendar)
   */
  public Timestamp getTimestamp(int columnIndex, Calendar cal) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getTimestamp(java.lang.String, java.util.Calendar)
   */
  public Timestamp getTimestamp(String columnName, Calendar cal) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getURL(int)
   */
  public URL getURL(int columnIndex) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#getURL(java.lang.String)
   */
  public URL getURL(String columnName) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateRef(int, java.sql.Ref)
   */
  public void updateRef(int columnIndex, Ref x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateRef(java.lang.String, java.sql.Ref)
   */
  public void updateRef(String columnName, Ref x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateBlob(int, java.sql.Blob)
   */
  public void updateBlob(int columnIndex, Blob x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateBlob(java.lang.String, java.sql.Blob)
   */
  public void updateBlob(String columnName, Blob x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateClob(int, java.sql.Clob)
   */
  public void updateClob(int columnIndex, Clob x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateClob(java.lang.String, java.sql.Clob)
   */
  public void updateClob(String columnName, Clob x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateArray(int, java.sql.Array)
   */
  public void updateArray(int columnIndex, Array x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * method not supported
   * 
   * @see java.sql.ResultSet#updateArray(java.lang.String, java.sql.Array)
   */
  public void updateArray(String columnName, Array x) throws SQLException {
    throw new SQLException("Method not supported");
  }

  /**
   * writes the resultset in a consistent manner
   */
  public String toString() {
    String ret = null;
    StringBuffer buf = new StringBuffer();
    try {
      ResultSetMetaData metaData = getMetaData();
      for (int i = 1; i <= metaData.getColumnCount(); i++) {
        buf.append("[");
        buf.append((metaData.getColumnName(i).toString() + "]"));
      }
      while (next()) {
        buf.append("\n");
        for (int i = 1; i <= metaData.getColumnCount(); i++) {
          buf.append("{").append(getString(metaData.getColumnName(i))).append("}");
        }
        buf.append("\n");
      }
    } catch (Exception e) {
      Write.exception(e);
      ret = "Error reading lines";
    }
    ret = buf.toString();
    return ret;
  }
}